@@ -1,9 +1,18 @@
+import io
 import json
 import os
+from logging import Formatter, StreamHandler
 from typing import Any, Dict, Optional, cast
 
 from chaoslib.run import EventHandlerRegistry, RunEventHandler
-from chaoslib.types import Configuration, Experiment, Journal, Secrets
+from chaoslib.types import (
+    Configuration,
+    Experiment,
+    Journal,
+    Schedule,
+    Secrets,
+    Settings,
+)
 from logzero import logger
 
 from chaosreliably import RELIABLY_HOST, get_session
@@ -16,17 +25,32 @@ class ReliablyHandler(RunEventHandler):  # type: ignore
         self,
         org_id: str,
         exp_id: str,
-        configuration: Configuration = None,
-        secrets: Secrets = None,
     ) -> None:
         RunEventHandler.__init__(self)
         self.org_id = org_id
         self.exp_id = exp_id
         self.exec_id = None
+
+        self.stream = io.StringIO()
+        self.log_handler = StreamHandler(stream=self.stream)
+        self.log_handler.setFormatter(
+            Formatter("[%(asctime)s] - %(levelname)s - %(message)s")
+        )
+
+        logger.addHandler(self.log_handler)
+
+    def running(
+        self,
+        experiment: Experiment,
+        journal: Journal,
+        configuration: Configuration,
+        secrets: Secrets,
+        schedule: Schedule,
+        settings: Settings,
+    ) -> None:
         self.configuration = configuration
         self.secrets = secrets
 
-    def started(self, experiment: Experiment, journal: Journal) -> None:
         try:
             result = create_run(
                 self.org_id,
@@ -42,6 +66,7 @@ class ReliablyHandler(RunEventHandler):  # type: ignore
                 extension = get_reliably_extension_from_journal(journal)
 
                 self.exec_id = payload["id"]
+                logger.debug(f"Reliably execution: {self.exec_id}")
 
                 host = self.secrets.get(
                     "host", os.getenv("RELIABLY_HOST", RELIABLY_HOST)
@@ -64,12 +89,19 @@ class ReliablyHandler(RunEventHandler):  # type: ignore
             )
 
     def finish(self, journal: Journal) -> None:
+        logger.removeHandler(self.log_handler)
+        self.log_handler.flush()
+
+        log = self.stream.getvalue()
+        self.stream.close()
+
         try:
             complete_run(
                 self.org_id,
                 self.exp_id,
                 self.exec_id,
                 journal,
+                log,
                 self.configuration,
                 self.secrets,
             )
@@ -95,11 +127,8 @@ def configure_control(
     secrets: Secrets = None,
     **kwargs: Any,
 ) -> None:
-    event_registry.register(
-        ReliablyHandler(
-            org_id, exp_id, configuration=configuration, secrets=secrets
-        )
-    )
+    logger.debug("Configure Reliably's experiment control")
+    event_registry.register(ReliablyHandler(org_id, exp_id))
 
 
 ###############################################################################
@@ -128,13 +157,14 @@ def complete_run(
     exp_id: str,
     execution_id: Optional[str],
     state: Journal,
+    log: str,
     configuration: Configuration,
     secrets: Secrets,
 ) -> Optional[Dict[str, Any]]:
     with get_session(configuration, secrets) as session:
         resp = session.put(
             f"/{org_id}/experiments/{exp_id}/executions/{execution_id}/results",
-            json={"result": json.dumps(state)},
+            json={"result": json.dumps(state), "log": log},
         )
         if resp.status_code != 200:
             logger.error("Failed to update results on server")
